﻿using System;

class SimplexMethod
{
    static void Main()
    {
        // Пример входных данных: коэффициенты ограничений в каноническом виде и коэффициенты целевой функции
        double[,] constraints = { { 4, 7, 1 }, { 8, 3, 1 }, { 9, 5, 1 } };
        double[] objectiveFunctionCoefficients = { -49, -51, -45 };

        // Максимизация целевой функции
        double[] optimalSolutionMax = SolveSimplexMethodMax(constraints, objectiveFunctionCoefficients);
        double objectiveValueMax = CalculateObjectiveFunctionValue(optimalSolutionMax, objectiveFunctionCoefficients);

        // Вывод результатов максимизации
        Console.WriteLine("Maximization:");
        Console.WriteLine("Optimal Objective Function Value: " + objectiveValueMax);
        Console.WriteLine("Optimal Solution Vector:");
        PrintSolution(optimalSolutionMax);

        // Минимизация целевой функции
        double[] objectiveFunctionCoefficientsNegated = NegateArray(objectiveFunctionCoefficients);
        double[] optimalSolutionMin = SolveSimplexMethodMin(constraints, objectiveFunctionCoefficientsNegated, objectiveFunctionCoefficients.Length);
        double objectiveValueMin = -CalculateObjectiveFunctionValue(optimalSolutionMin, objectiveFunctionCoefficientsNegated);

        // Вывод результатов минимизации
        Console.WriteLine("\nMinimization:");
        Console.WriteLine("Optimal Objective Function Value: " + objectiveValueMin);
        Console.WriteLine("Optimal Solution Vector:");
        PrintSolution(optimalSolutionMin);
    }

    static double[] SolveSimplexMethodMax(double[,] constraints, double[] objectiveFunctionCoefficients)
    {
        int numVariables = objectiveFunctionCoefficients.Length;
        int numConstraints = constraints.GetLength(0);

        // Добавляем дополнительные переменные и выражаем их через базисные переменные
        double[,] tableau = new double[numConstraints + 1, numVariables + numConstraints + 1];
        for (int i = 0; i < numConstraints; i++)
        {
            for (int j = 0; j < numVariables; j++)
            {
                tableau[i, j] = constraints[i, j];
            }
            tableau[i, numVariables + i] = 1;
            tableau[i, numVariables + numConstraints] = constraints[i, numVariables];
        }

        // Добавляем целевую функцию
        for (int j = 0; j < numVariables; j++)
        {
            tableau[numConstraints, j] = -objectiveFunctionCoefficients[j];
        }
        tableau[numConstraints, numVariables + numConstraints] = 1;

        // Применяем симплекс метод
        while (true)
        {
            int pivotColumn = FindEnteringVariable(tableau);
            if (pivotColumn == -1) break;

            int pivotRow = FindLeavingVariable(tableau, pivotColumn);
            if (pivotRow == -1) throw new Exception("The solution is unbounded.");

            Pivot(tableau, pivotRow, pivotColumn);
        }

        // Возвращаем оптимальное решение
        double[] optimalSolution = new double[numVariables];
        for (int i = 0; i < numConstraints; i++)
        {
            if (tableau[i, numVariables + numConstraints] >= 0) continue;
            for (int j = 0; j < numVariables; j++)
            {
                optimalSolution[j] = tableau[i, j];
            }
            break;
        }
        return optimalSolution;
    }

    static double[] SolveSimplexMethodMin(double[,] constraints, double[] objectiveFunctionCoefficients, int numVariables)
    {
        int numConstraints = constraints.GetLength(0);

        // Добавляем дополнительные переменные и выражаем их через базисные переменные
        double[,] tableau = new double[numConstraints + 1, numVariables + numConstraints + 1];
        for (int i = 0; i < numConstraints; i++)
        {
            for (int j = 0; j < numVariables; j++)
            {
                tableau[i, j] = constraints[i, j];
            }
            tableau[i, numVariables + i] = 1;
            tableau[i, numVariables + numConstraints] = constraints[i, numVariables];
        }

        // Добавляем целевую функцию
        for (int j = 0; j < numVariables; j++)
        {
            tableau[numConstraints, j] = objectiveFunctionCoefficients[j];
        }
        tableau[numConstraints, numVariables + numConstraints] = 1;

        // Применяем симплекс метод
        while (true)
        {
            int pivotColumn = FindEnteringVariable(tableau);
            if (pivotColumn == -1) break;

            int pivotRow = FindLeavingVariable(tableau, pivotColumn);
            if (pivotRow == -1) throw new Exception("The solution is unbounded.");

            Pivot(tableau, pivotRow, pivotColumn);
        }

        // Возвращаем оптимальное решение
        double[] optimalSolution = new double[numVariables];
        for (int i = 0; i < numConstraints; i++)
        {
            if (tableau[i, numVariables + numConstraints] >= 0) continue;
            for (int j = 0; j < numVariables; j++)
            {
                optimalSolution[j] = tableau[i, j];
            }
            break;
        }
        return optimalSolution;
    }

    static int FindEnteringVariable(double[,] tableau)
    {
        int numVariables = tableau.GetLength(1) - 1;
        double maxCoefficient = 0;
        int pivotColumn = -1;
        for (int j = 0; j < numVariables; j++)
        {
            if (tableau[tableau.GetLength(0) - 1, j] > maxCoefficient)
            {
                maxCoefficient = tableau[tableau.GetLength(0) - 1, j];
                pivotColumn = j;
            }
        }
        return pivotColumn;
    }

    static int FindLeavingVariable(double[,] tableau, int pivotColumn)
    {
        int numConstraints = tableau.GetLength(0) - 1;
        int pivotRow = -1;
        double minRatio = double.MaxValue;
        for (int i = 0; i < numConstraints; i++)
        {
            if (tableau[i, pivotColumn] <= 0) continue;
            double ratio = tableau[i, tableau.GetLength(1) - 1] / tableau[i, pivotColumn];
            if (ratio < minRatio)
            {
                minRatio = ratio;
                pivotRow = i;
            }
        }
        return pivotRow;
    }

    static void Pivot(double[,] tableau, int pivotRow, int pivotColumn)
    {
        int numRows = tableau.GetLength(0);
        int numCols = tableau.GetLength(1);

        double pivotValue = tableau[pivotRow, pivotColumn];
        for (int j = 0; j < numCols; j++)
        {
            tableau[pivotRow, j] /= pivotValue;
        }
        for (int i = 0; i < numRows; i++)
        {
            if (i == pivotRow) continue;
            double multiplier = tableau[i, pivotColumn];
            for (int j = 0; j < numCols; j++)
            {
                tableau[i, j] -= multiplier * tableau[pivotRow, j];
            }
        }
    }

    static double[] NegateArray(double[] array)
    {
        double[] negatedArray = new double[array.Length];
        for (int i = 0; i < array.Length; i++)
        {
            negatedArray[i] = -array[i];
        }
        return negatedArray;
    }

    static double CalculateObjectiveFunctionValue(double[] solution, double[] objectiveFunctionCoefficients)
    {
        double value = 0;
        for (int i = 0; i < solution.Length; i++)
        {
            value += solution[i] * objectiveFunctionCoefficients[i];
        }
        return value;
    }

    static void PrintSolution(double[] solution)
    {
        for (int i = 0; i < solution.Length; i++)
        {
            Console.WriteLine("x[" + (i + 1) + "] = " + solution[i]);
        }
    }
}

